#include "MatrixFile.h"
using namespace std;


 
/**
* Finds the next inststance of a string in a given input stream.
* The stream is left with its get pointer pointing just after the instance of the
* string if one was found, or just after the end of the stream if one wasn't.
* @param in the stream in which to seek the string
* @param sought the string to seek
* @return the position in the stream where the found instance of the string started or just after the end of the stream if the string was not found
*/
streampos findNextInStream(istream & in, string sought)
{
	string::iterator cf = sought.begin(); //an iterator pointing to the next character to be checked in the sought string - all previous characters in the string should just have been found
	while (in.good() && cf != sought.end())
	{
		char c;
		in.get(c);
		if (c == *cf)
		{
			cf++;
		}
		else
		{
			cf = sought.begin();
		}
	}

	/*if (in.fail())
	{
		throw istream::fail;
	}*/
	
	if (cf == sought.end()) //if we found the sought string
	{
		return in.tellg() - streamoff(sought.length());
	}
	else
	{
		return in.tellg();
	}
}


bool MatrixFile::save(const MMatrix<double> & matrix)
{
	ofstream fout(m_filePath.c_str(), ios_base::app | ios_base::out);
	fout.precision(FLOAT_SAVE_PRECISION);
	fout << endl;
	fout << m_label << " " << m_num << endl;
	matrix.writeToStream(fout);
	fout << endl;
	return !(fout.bad() || fout.fail());
}

bool MatrixFile::save(string label, string value)
{
	ofstream fout(m_filePath.c_str(), ios_base::app | ios_base::out);
	fout << label << ": " << value << endl;
	return !(fout.bad() || fout.fail());
}

bool MatrixFile::save(string value)
{
	ofstream fout(m_filePath.c_str(), ios_base::app | ios_base::out);
	fout << value << endl;
	return !(fout.bad() || fout.fail());
}

bool MatrixFile::save(string label, int value)
{
	ofstream fout(m_filePath.c_str(), ios_base::app | ios_base::out);
	fout << label << ": " << value << endl;
	return !(fout.bad() || fout.fail());
}

bool MatrixFile::save(string label, double value)
{
	ofstream fout(m_filePath.c_str(), ios_base::app | ios_base::out);
	fout.precision(FLOAT_SAVE_PRECISION);
	fout << label << ": " << value << endl;
	return !(fout.bad() || fout.fail());
}


bool MatrixFile::loadFirstMatrix(MMatrix<double> & matrix)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	findNextInStream(fin, m_label);
	if (fin.good())
	{
		fin >> m_num;
		fin >> matrix;
	}
	else
	{
		return false;
	}
	return !(fin.bad() || fin.fail());
}

string MatrixFile::loadFirstString(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	string ret;
	findNextInStream(fin, label + ":");
	if (fin.good())
	{
		getline(fin, ret);
	}
	return ret;
}

int MatrixFile::loadFirstInt(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	int ret = 0;
	findNextInStream(fin, label + ":");
	if (fin.good())
	{
		fin >> ret;
	}
	return ret;
}

double MatrixFile::loadFirstDouble(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	double ret = 0.;
	findNextInStream(fin, label + ":");
	if (fin.good())
	{
		fin >> ret;
	}
	return ret;
}

bool MatrixFile::findLast(istream & in, string label)
{
	if (label == "")
	{
		return false;
	}
	streampos startPos = in.tellg();
	streampos lastPos = startPos;
	while (in.good())
	{
		lastPos = in.tellg();
		findNextInStream(in, label);
	}
	if (in.bad() || !in.eof() 
		   || lastPos == (streampos)-1
		   || startPos == lastPos) //if there was some kind of error or we didn't find the label
	{
		return false;
	}
	else
	{
		in.clear(); //clear eof flag so we can use seekg
		in.seekg(lastPos);
		return true;
	}
}


bool MatrixFile::loadLastMatrix(MMatrix<double> & matrix)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	if (findLast(fin, m_label))
	{
		fin >> m_num;
		fin >> matrix;
	}
	else
	{
		return false;
	}
	return !(fin.bad() || fin.fail());
}


string MatrixFile::loadLastString(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	string ret;
	if (findLast(fin, label + ":"))
	{
		getline(fin, ret);
	}
	return ret;
}

int MatrixFile::loadLastInt(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	int ret = 0;
	if (findLast(fin, label + ":"))
	{
		fin >> ret;
	}
	return ret;
}

double MatrixFile::loadLastDouble(string label)
{
	ifstream fin(m_filePath.c_str(), ios_base::in);
	double ret = 0.;
	if (findLast(fin, label + ":"))
	{
		fin >> ret;
	}
	return ret;
}

