
#ifndef MATRIXFILE_H
#define MATRIXFILE_H

#include <string>
#include <fstream>
#include "MMatrix.h"
using namespace std;

/**
 * Class to ease the saving of a sequence of MMatrices of doubles and accompanying information to file
 */
class MatrixFile
{
	public:
		MatrixFile(string filePath): m_filePath(filePath), m_num(0){}
		~MatrixFile(){}
		
		//! Sets the label that is written before each matrix
		void setMatrixLabel(string label) {m_label = label;}
		//! Sets the number that will be written with the label above the next matrix
		void setMatrixNum(int num) {m_num = num;}
		const char * getMatrixLabel() {return m_label.c_str();}
		const char * getFilePath() {return m_filePath.c_str();}
		//! Gets the number that will be written with the label above the next matrix
		int getMatrixNum() {return m_num;}
		//! Increases by 1 the number that will be written with the label above the next matrix
		void incrementNum() {m_num++;}
		
		/**
		* Saves the matrix given to file preceeded by a label and number.
		* @return whether saving succeded
		*/
		bool save(const MMatrix<double> & matrix);
		/**
		 * Saves the string given to file
		 * @return whether saving succeded
		 */
		bool save(string value);
		/**
		 * Saves the value given to file preceeded by the label (followed by a colon)
		 * @return whether saving succeded
		 */
		bool save(string label, string value);
		/**
		 * Saves the value given to file preceeded by the label (followed by a colon)
		 * @return whether saving succeded
		 */
		bool save(string label, int value);
		/**
		 * Saves the value given to file preceeded by the label (followed by a colon)
		 * @return whether saving succeded
		 */
		bool save(string label, double value);
		
		
		/**
		 * @param matrix into this MMatrix, is loaded the MMatrix found following the first instance of the matrix label.
		 * The matrix number by the label is loaded as the current matrix number.
		 * @return whether loading succeeded
		 */
		bool loadFirstMatrix(MMatrix<double> & matrix);
		/**
		* @return the string found in the file following the first instance of the specified label
		*	The string is taken as all text from the label to the end of the line
		* 	If loading fails, "" is returned.
		*/
		string loadFirstString(string label);
		/**
		 * @return the int found in the file following the first instance of the specified label
		 * 		If loading fails, 0 is returned.
		 */
		int loadFirstInt(string label);
		/**
		 * @return the double found in the file following the first instance of the specified label
		 * 		If loading fails, 0. is returned.
		 */
		double loadFirstDouble(string label);
		
		/**
		* @param matrix into this MMatrix, is loaded the MMatrix found following the last instance of the matrix label.
		 * The matrix number by the label is loaded as the current matrix number.
		* @return whether loading succeeded
		*/
		bool loadLastMatrix(MMatrix<double> & matrix);
		/**
		 * @return the string found in the file following the last instance of the specified label.
		 *	The string is taken as all text from the label to the end of the line.
		 * 	If loading fails, "" is returned.
		 */
		string loadLastString(string label);
		/**
		 * @return the int found in the file following the last instance of the specified label
		 * 		If loading fails, 0 is returned.
		 */
		int loadLastInt(string label);
		/**
		 * @return the double found in the file following the last instance of the specified label
		 * 		If loading fails, 0. is returned.
		 */
		double loadLastDouble(string label);
		
		//The maximum number of decimal places to which floating point values are saved, including the doubles in MMatrices
		const static streamsize FLOAT_SAVE_PRECISION = 20;

	private:
		string m_label;
		string m_filePath;
		int m_num;
		
		static bool findLast(istream & in, string label);
};

#endif //MATRIXFILE_H
