

#include "Polygon.h"
#include <sstream>
#include <iostream>
using namespace std;

void Polygon::addEdge(int x1d, int x2d, int eLen)
{
	m_x1d.push_back(x1d);
	m_x2d.push_back(x2d);
	m_eLen.push_back(eLen);
	m_x1.push_back(m_x1.back() + eLen * x1d);
	m_x2.push_back(m_x2.back() + eLen * x2d);
}


bool Polygon::parse(const string & s)
{
	//DEBUG - cout << s << endl;
	istringstream ss(s);
	return parse(ss);
}

bool Polygon::parse(istream & in)
{
	ios_base::fmtflags streamOrigFlags = in.flags(); //store the current format flags of the stream given to restore later
	in >> skipws; //ignore whitespace (spaces, tabs, newlines etc) in string
	Polygon tempPoly;
	char openingBracket = '\0';
	char closingBracket = '\0';
	char comma = '\0';
	int t1 = 0;
	int t2 = 0;
	int tLen = 0;
	bool error = false;
	
	//parse initial point
	in >> openingBracket >> t1 >> comma >> t2 >> closingBracket;
	//DEBUG - cout << "Initial Point " << t1 << "    " << t2 << " " << openingBracket << closingBracket << comma << in.good();
	if (in.good()
		   && openingBracket == '(' 
		   && closingBracket == ')' 
		   && comma == ',' 
		   && t1 >= 0 
		   && t2 >= 0)
	{
		tempPoly.setInitialPoint(t1,t2);
	}
	else
	{
		error = true;
	}
	
	//parse edge vectors and multiplicities
	in >> openingBracket;
	while(!error && !in.eof() && openingBracket != '#')
	{
		in >> t1 >> comma >> t2 >> closingBracket >> tLen;
		//DEBUG - cout << "Edge " << t1 << "," << t2 << "|" << tLen << "|" << openingBracket << closingBracket << comma << in.good() << in.eof() << endl;
		if ((in.good() || in.eof())
		   && openingBracket == '(' 
		   && closingBracket == ')' 
		   && comma == ',' 
		   && !(t1 == 0 && t2 == 0)
		   && tLen > 0)
		{
			
			tempPoly.addEdge(t1, t2, tLen);
		}
		else
		{
			error = true;
		}
		in >> openingBracket;
	}
	
	//DEBUG - cout << "checking";
	if (!error)
	{
		if (tempPoly.checkValid())
		{
			//DEBUG - cout << "valid";
			(*this) = tempPoly;
		}
		else
		{
			error = true;
		}
	}
	
	in.flags(streamOrigFlags);
	return !error;
}


bool Polygon::checkValid() const
{
	bool numVertsConsistant = (m_x1.size() == m_x2.size() 
								&& m_x1.size() == m_x1d.size() + 1
								&& m_x1.size() == m_x2d.size() + 1
								&& m_x1.size() == m_eLen.size() + 1);
	bool closed = (m_x1.front() == m_x1.back() && m_x2.front() == m_x2.back());
	return numVertsConsistant && closed && m_x1.size() > 2;
}

string Polygon::toEdgeString() const
{
	ostringstream ss;
	if (!checkValid())
	{
		return ss.str();
	}
	ss << '(' << m_x1[0] << ',' << m_x2[0] << ')';
	for (int i = 0; i < m_x1d.size(); i++)
	{
		ss << " (" << m_x1d[i] << ',' << m_x2d[i] << ')' << m_eLen[i];
	}
	return ss.str();
}

bool Polygon::contains(double x, double y) const
{
	bool inside = true;
	for (int bet = 0;bet <= numVerts() - 1;bet++)
	{
		double lambda = (double) m_x1d[bet] * (y - (double) m_x2[bet])
					- (double) m_x2d[bet] * (x - (double) m_x1[bet]);
		if (lambda < 0)
		{
			inside = false;
		}
	}
	return inside;
}






void Circle::set(double x, double y, double radius)
{
	m_x = x;
	m_y = y;
	m_r = radius;
	m_rSquared = pow2(radius);
}

bool Circle::contains(double x, double y) const
{
	double distFromCentreSq = pow2(m_x - x) + pow2(m_y - y);
	return distFromCentreSq < m_rSquared;
}

string Circle::toString() const
{
	ostringstream ss;
	ss << "(" << m_x << "," << m_y << ")" << m_r;
	return ss.str();
}

