
#ifndef POLYGON_H
#define POLYGON_H

#include <string>
#include <vector>
#include "maths_util.h"
using namespace std;

/**
 * Represents a polygon on the xy plane.
 */
class Polygon
{
	public:
		Polygon() {m_x1.push_back(0); m_x2.push_back(0);}
		~Polygon(){}
		
		void setInitialPoint(int pX1, int pX2) {m_x1.at(0) = pX1; m_x2.at(0) = pX2;}
		void addEdge(int x1d, int x2d, int eLength);
		
		//getter methods
		inline const vector<int> & x1() const {return m_x1;}
		inline const vector<int> & x2() const {return m_x2;}
		inline const vector<int> & x1d() const {return m_x1d;}
		inline const vector<int> & x2d() const {return m_x2d;}
		inline const vector<int> & eLen() const {return m_eLen;}
		inline const int numVerts() const {return m_x1d.size();}
		
		//! return whether the vectors describing the polygon are consistant, the polygon is closed, and has more than 2 sides
		bool checkValid() const;
		/**
		* set the polygon according to a given polygon string if this describes a valid polygon.
		* @return whether the polygon string described a valid polygon (checked by checkValid())
		*/
		bool parse(const string & s);
		/**
		 * set the polygon according to a polygon string extracted from the given stream if this describes a valid polygon.
		 * @return whether the polygon string described a valid polygon (checked by checkValid())
		 */
		bool parse(istream & in);
		//! @return the polygon string for the polygon
		string toEdgeString() const;
		
		//! @return whether the polygon contains the point (x,y)
		bool contains(double x, double y) const;
	private:
		vector<int> m_x1; //[NV+1];//x1 coordinate of vertex
		vector<int> m_x2; //[NV+1];//x2 coordinate of vertex
		vector<int> m_x1d; //[NV];//x1 component of primitive difference vector
		vector<int> m_x2d; //[NV]; //x2 component of primitive difference vector
		vector<int> m_eLen; //[NV];//"length" of edge--number of integer points minus 1
};


/**
 * Represents a circle on the xy plane.
 */
class Circle
{
	public:
		Circle(): m_x(0), m_y(0), m_r(0) {m_rSquared = pow2(m_r);}
		Circle(double x, double y, double radius): m_x(x), m_y(y), m_r(radius) {m_rSquared = pow2( m_r);}
		~Circle() {}
		
		double getX() const {return m_x;}
		double getY() const {return m_y;}
		double getRadius() const {return m_r;}
		
		void set(double x, double y, double radius);
		
		//! @return whether the circle contains the point (x,y)
		bool contains(double x, double y) const;
		//! @return a string representation of the circle - the coordinates of the centre followed by the radius.
		string toString() const;
	private:
		double m_x;
		double m_y;
		double m_r;
		double m_rSquared;
};

#endif //POLYGON_H
