
 
#include "plot_util.h"
#include <stdlib.h> //for exit()
#include <iostream>
#include <stdio.h>
#include <string.h>
using namespace std;


char isNumeric(char c)
{
	return c >= '0' && c <= '9';
}


int init_plotsession(const char* display_type, const char* size, FILE* file)
{
  /*DEBUG fprintf(stdout, "display type: %s    size: %s\n", display_type, size);*/
  if (!display_type || !size)
  {
    fprintf(stderr,"init_plotsession must be given null display_type or size\n");
    exit(1);
  }
  int handle, return_value;
  char a_size[100];
  strncpy(a_size, size, 99); /* make a copy as pl_parampl cannot cope with constant character array */
  if (isNumeric(size[0]))
  {
    return_value=pl_parampl("BITMAPSIZE", a_size);
  }
  else
  {
    return_value=pl_parampl("PAGESIZE", a_size);
  }
  if(return_value < 0)
  {
    fprintf(stderr,"The plotter does not accept the parameters specified.\n");
    exit(1);
  }
  if (strcmp(display_type, "X") == 0 || strcmp(display_type, "x") == 0 || !(file) )
  {
  	handle=pl_newpl(display_type, stdin, stdout, stderr);
  }
  else
  {
    handle=pl_newpl(display_type, stdin, file, stderr);
  }

  if(handle < 0)
  {
    fprintf(stderr,"Can't create plotter.\n");
    exit(1);
  }
  return_value=pl_selectpl(handle);
  if(return_value < 0)
  {
    fprintf(stderr,"Plotter does not exist\n");
    exit(1);
  }
  return_value=pl_openpl();
  if(return_value < 0)
  {
    fprintf(stderr,"Plotter can't be opened.\n");
    exit(1);
  }
  return handle;
}


int end_plotsession(int handle) /* closes the plotter -> 0 if OK */  {
   int err_status=0, return_value;
   return_value=pl_selectpl(handle);
   if(return_value<0)
   {
	   fprintf(stderr,"Plotter to be deleted could not be selected!\n");
	   err_status +=1;
   }
   return_value= pl_closepl(); /* close the plotter session*/
   if(return_value<0)
   {
     fprintf(stderr,"Problem closing the plotter\n");
     /* no exit, because we try to delete the plotter */
	 err_status +=1<<2;
   }
   return_value=pl_selectpl(0);
   if(return_value<0)
   {
     fprintf(stderr,"Default Plotter could not be selected!\n");
     err_status +=1<<3;
   }
   /* deleting our plotter */
   return_value=pl_deletepl (handle);
   if(return_value<0)
   {
     fprintf(stderr,"Could not delete plotter!\n");
     err_status +=1<<4;
   }
   return err_status;
}





CPlotter::CPlotter(CPlotter & other)
{
	if (m_handle >= 0)
	{
		if(!close())
		{
			cout << " so cannot continue copying plotter." << endl;
			return;
		}
	}
	
	m_handle = other.m_handle;
	other.m_handle = -1;
	m_file = other.m_file;
	other.m_file = NULL;
}

CPlotter::~CPlotter()
{
	close();
}

void CPlotter::open(const string & display_type, const string & size, const string & fileName)
{
	if(!close())
	{
		cout << "so cannot continue opening new plotter." << endl;
		exit(1);
	}
	
	string fullFileName(fileName);
	if (fileName != "" && display_type != "X" && display_type != "x")
	{
		fullFileName.append(".").append(display_type);
		m_file = fopen(fullFileName.c_str(), "w");
		if (!m_file)
		{
			cout << "failed to open file to save plot" << endl;
			exit(1);
		}
	}
	else
	{
		m_file = NULL;
		fullFileName = "";
	}
	
	m_displayType = display_type;
	m_size = size;
	m_fileName = fullFileName;
	
	m_handle = init_plotsession(display_type.c_str(), size.c_str(), m_file);
}

void CPlotter::open(char displayTypeChar, const string & fileName)
{
	string display_type;
	string size;
	switch (displayTypeChar)
	{
		case 'x':
			display_type = "X";
			size = "800x800";
			break;
		case 'p':
			display_type = "png";
			size = "600x600";
			break;
		case 'g':
			display_type = "gif";
			size = "600x600";
			break;
		case 's':
			display_type = "ps";
			size = "a4";
			break;
		default:
			cout << "invalid input choice" << endl;
			exit(1);
	}
	
	open(display_type, size, fileName);
}

bool CPlotter::close()
{
	int plotSessionStatus = 0;
	if (m_handle >= 0)
	{
		plotSessionStatus = end_plotsession(m_handle);
	}
	int fileStatus = 0;
	if (m_file)
	{
		fileStatus = fclose(m_file);
		if (fileStatus != 0)
		{
			cout << "failed to close plotter file" << endl;
		}
	}
	return (fileStatus == 0) && (plotSessionStatus == 0);
}

bool CPlotter::select() const
{
	if (m_handle < 0)
	{
		cout << "Cannot select an invalid plotter" << endl;
		return false;
	}
	else
	{
		int return_value=pl_selectpl(m_handle);
		if(return_value < 0)
		{
			cout << "Failed to select plotter" << endl;
			return false;
		}
		return true;
	}
}

void CPlotter::printDetails() const
{
	if (m_handle < 0)
	{
		cout << "libplot plotting not set up" << endl;
	}
	else
	{
		if (m_displayType == "X" || m_displayType == "x") 
		{
			cout << "libplot plotting to be displayed in a window" << endl;
		}
		else 
		{
			cout << "libplot plotting to be saved as a " << m_displayType << endl;
			cout << " in file " << m_fileName << endl;
		}
	}
}

int CPlotter::getHandle() const
{
	return m_handle;
}

FILE * CPlotter::getFile() const
{
	return m_file;
}

const string & CPlotter::getDisplayType() const
{
	return m_displayType;
}

const string & CPlotter::getSize() const
{
	return m_size;
}

const string & CPlotter::getFileName() const
{
	return m_fileName;
}

