#include "plotting.h"
#include "maths_util.h"
#include <iostream>
using namespace std;

void sixcolourplot(double var, double cd1, double cd2, double a1, double a2, double a3, double a4, double a5)
{
	if (var > a5)
	{
		pl_pencolorname ("red");
		pl_fmarker(cd1, cd2, 3, .1);
	}

	if  (var < a5 && var >= a4)
	{
		pl_pencolorname ("orange");
		pl_fmarker(cd1, cd2, 3, .1);
	}
	if (var < a4 && var >= a3)
	{
		pl_pencolorname ("yellow");
		pl_fmarker(cd1, cd2, 3, .1);
	}
	if (var < a3 && var >= a2)
	{
		pl_pencolorname ("green");
		pl_fmarker(cd1, cd2, 3, .1);
	}
	if (var < a2 && var >= a1)
	{
		pl_pencolorname ("blue");
		pl_fmarker(cd1, cd2, 3, .1);
	}
	if (var < a1)
	{
		pl_pencolorname("pink");
		pl_fmarker(cd1, cd2, 3, .1);
	}
	//	pl_fontname("Helvetica");
	//	pl_fontsize(-1);
	//	pl_pencolorname("Black");
	//	pl_fmove(4.25,.45);
	//	pl_label("red >.06");
	//	pl_fmove(4.25,.3);
	//	pl_label("orange .03-.06");
	//	pl_fmove(4.25,0.15);
	//	pl_label("yellow 0.0-.03");
	//	pl_fmove(4.25,0.0);
	//	pl_label("green -.03-.0");
	//	pl_fmove(4.25,-.15);
	//	pl_label("blue .-.06- -.03");
	//	pl_fmove(4.25,-.3);
	//	pl_label("purple<.-.06");

}

void simpleContourPlot(vector<Vector3> results, int numContours)
{
	MinMax minMax;
	for(int i = 0; i < results.size(); ++i)
	{
		minMax.contend(results[i].getZ());
	}
	double maxVal = minMax.max();
	double minVal = minMax.min();
	double range = maxVal - minVal;
	double ratio = maxVal / minVal;
	double contStep = range / double (numContours);
	cout << "Contour plotted values" << endl;
	cout << "min:" << minVal << " max:" << maxVal << endl;
	cout << "ratio of max/min: " << ratio << endl;
	
	
	vector<double> contourVals; //points with values around those in this list are plotted
	double contourVal = minVal + contStep/2;
	for (int i = 0; i < numContours; i++)
	{
		contourVals.push_back(contourVal);
		cout << "contour " << i << ": " << contourVal << endl;
		contourVal += contStep;
	}
	//DEBUG - for (int i = 0; i < contVal.size(); i++) cout << contVal[i] << endl;
	
	
	double contourTol = range / 60; //how far from a contourVal the value of a point may be for that point to be plotted
	cout << "Contour Tolerance: " << contourTol << endl;
	
	vector<Vector3>::iterator resEnd = results.end();
	for(vector<Vector3>::iterator i = results.begin(); i != resEnd; ++i)
	{
		simpleContourPlotPoint(i->getZ(), i->getX(), i->getY(), contourTol, contourVals);
	}
}



void simpleContourPlotPoint(double var, double x1, double x2, double tolerance, const vector<double> & contours)
{
	vector<double>::const_iterator c = contours.begin();
	bool done = false;
	//DEBUG - cout << x1 << "," << x2 << "   " << var << endl;
	int colourStep = COLOURMAX / ( contours.size() + 1 );
	while( c != contours.end() && !done)
	{
		if (var < *c + tolerance && var > *c - tolerance)
		{
			pl_pencolor(colourStep * (c-contours.begin()), 0, 0);
			pl_fmarker(x1, x2, 1, .1);
			done = true;
		}
		c++;
	}
}

void autoSixColourPlot(vector<Vector3> results)
{
	const string colourName[] = {"red", "orange", "yellow", "green", "blue", "magenta"};
	const int numColours = 6;
	
	MinMax minMax;
	for(int i = 0; i < results.size(); ++i)
	{
		minMax.contend(results[i].getZ());
	}
	double maxVal = minMax.max();
	double minVal = minMax.min();
	double range = maxVal - minVal;
	double ratio = maxVal/minVal;
			
	cout << "Colour plot values" << endl;
	cout << "min:" << minVal << " max:" << maxVal << endl;
	cout << "ratio of max/min: " << ratio << endl;
			
	vector<double> divisionVals; //values on different sides of these values are different colours
	double step = range / double(numColours);
	double divisionVal = minVal;
	for (int i = 0; i < numColours; i++)
	{
		divisionVals.push_back(divisionVal);
		divisionVal += step;
	}
	divisionVals.push_back(maxVal); //max value may not exactly equal minVal + numColours * step
	
	cout << endl;
	for (int i = 0; i < numColours; i++)
	{
		cout << colourName[i] << ": " << divisionVals[i]
				<< " to " << divisionVals[i+1] << endl;
	}
	
	for(int i = 0; i < results.size(); ++i)
	{
		int j = 0;
		bool done = false;
		while(j < numColours && !done)
		{
			double val = results[i].getZ();
			if (val >= divisionVals[j] && val <= divisionVals[j + 1])
			{
				pl_pencolorname(colourName[j].c_str());
				pl_fmarker(results[i].getX(), results[i].getY(), 1, .05);
				done = true;
			}
			j++;
		}
	}
}



void drawPolygon(Polygon pg)
{
	for (int a = 0;a < pg.numVerts();a++)
	{
		pl_fline (pg.x1()[a], pg.x2()[a], pg.x1()[a+1], pg.x2()[a+1]);
	}
}


void draw(Circle aCircle)
{
	pl_fcircle(aCircle.getX(), aCircle.getY(), aCircle.getRadius());
}

