

#include <list>
#include <fstream>
#include <cmath>
#include <iostream>
#include <algorithm>
#include "xyzFileSave.h"
using namespace std;


void removeClosePoints(vector<Vector3> & results, const double elimCloseness, vector<Vector3> & removed)
{
	if (elimCloseness > 0)
	{
		list<Vector3> rList(results.begin(), results.end());
		rList.sort();
			
		for(list<Vector3>::iterator i = rList.begin(); i != rList.end(); ++i)
		{
			list<Vector3>::iterator j = i;
			++j;
			double maxPossibleX = i->getX() + elimCloseness;
			while(j->getX() < maxPossibleX && j != rList.end())
			{
				if ( abs(i->getX() - j->getX()) < elimCloseness 
							&& abs(i->getY() - j->getY()) < elimCloseness )
				{
					removed.push_back(*j);
					j = rList.erase(j);
				}
				else
				{
					++j;
				}
			}
		}
		vector<Vector3> tempVect(rList.begin(), rList.end());
		swap(results, tempVect); //swap is very quick -  swaps pointers to the data not the data itself
	}
}



void xyzSave(vector<Vector3> results, const string & xyzFileName, const double elimCloseness, bool randomise, char separator)
{	
	int totalPoints = results.size();
	
	vector<Vector3> removed;
	removeClosePoints(results, elimCloseness, removed);
	
	if (randomise)
	{
		//DEBUG - cout << "randomising points" << endl;
		random_shuffle(results.begin(), results.end());
	}
	
	ofstream xyzFile(xyzFileName.c_str(), ios_base::out | ios_base::trunc);
	if (xyzFile.is_open())
	{
		xyzFile.precision(XYZ_FILE_PRECISION);
	
		vector<Vector3>::iterator resEnd = results.end();
		for(vector<Vector3>::iterator i = results.begin(); i != resEnd; ++i)
		{
			double output = i->getZ();
			if (!isnan(output) && !isinf(output))
			{
				xyzFile << i->getX() << separator;
				xyzFile << i->getY() << separator;
				xyzFile << output << endl;
			}
		}
		xyzFile.close();
		cout << "Saved xyz file with " << results.size();
		cout << "/" << totalPoints << " points.\n";
	}
	else
	{
		cout << "Failed to open xyz file" << endl;
	}
	
}

