

#ifndef XYZFILESAVE_H
#define XYZFILESAVE_H

#include <vector>
#include <string>


/**
 * Simple class representing a point in 3D space.
 */
class Vector3
{
	//TODO far more functionality could be added to this class
	public:
		Vector3(): m_x(0), m_y(0), m_z(0) {}
		Vector3(double x, double y, double z): m_x(x), m_y(y), m_z(z) {}
		~Vector3() {}
		
		double getX() const {return m_x;}
		double getY() const {return m_y;}
		double getZ() const {return m_z;}
		
		bool operator<(const Vector3 & rhs) const {return m_x < rhs.getX();}
	private:
		double m_x;
		double m_y;
		double m_z;
};


/**
 * Saves an xyz file suitable for xd3d or a csv file suitable for matlab
 * each line of the file has the x, y and z values of a point to plot
 * @param results		a vector of the results to save
 * @param xyzFileName	the name of the file to save (including extension)
 * @param elimCloseness	if this is greater than 0, only one of any pair of points closer than this in the x or y directions is saved.
 * @param randomise 	whether or not to write the results to file in a random order
 * @param separator 	the separating character between x, y and z values in the file
 */
void xyzSave(std::vector<Vector3> results, const std::string & xyzFileName, 
			 const double elimCloseness = 0, bool randomise = false, char separator = ' ');
const int XYZ_FILE_PRECISION = 15; //< the maximum number of digits of the values to write to file

/**
* Remove points from a vector that are overly close in the xy plane
* @param results		the vector to be processed
* @param elimCloseness	one of any two points closer than this in both x and y directions are removed
* @param removed		the removed points are added to this vector
*/
void removeClosePoints(std::vector<Vector3> & results, const double elimCloseness, std::vector<Vector3> & removed);

#endif //XYZFILESAVE_H
